# Discussion

## Mystery Sort

**Complete the following table. Each pass through the mystery sort corresponds to one complete iteration of the do-while loop.**

| Indices | [0]  | [1]  | [2]  | [3]  | [4]  | [5]  | [6]  | [7]  |
| ------- | :---: | :---: | :---: | :---: | :---: | :---: | :---: | :---: |
| Input    | 14   | 10   | 23   | 34   | 6    | 17   | 50   | 14   |
| 1st pass | 10   | 6    | 14   | 23   | 14   | 17   | 34   | 50   |
| 2nd pass | 6    | 10   | 14   | 14   | 17   | 23   | 34   | 50   |

**Describe (briefly) the strategy (algorithm) behind the mystery sort. (One or two paragraphs at most.) Elaborate on its asymptotic runtime and space complexity in the best and worst cases. 
Make sure to note what are the best/worse case scenarios for the mystery sort.**

The mystery sort is known as bi-directional bubble sort, or cocktail sort.

The first rightward pass will shift the largest element to the right end, and the following leftward pass will shift the smallest element to the left end. The following complete passes will shift the second largest/smallest elements to their correct places. After i passes, the first i and the last i elements in the list are in their correct positions, and do not need to be checked.

Similar to Bubble sort, we use early stop to short circuit the process when no swap is performed (in either direction).

Cocktail sort is a slight improvement over bubble sort. Instead of repeatedly passing through the list from left to right, it passes alternately in both directions. Bubble sort only passes through the list in one direction and therefore can only move items backward one step each iteration. That said, the worst-case performance is still $O(n^2)$. The best-case case performance is $O(n)$; cocktail sort's best case is when the input is sorted or nearly sorted! The worst case is when the input is in (nearly) reverse sorted order.

## Max's Sorting Strategy

**Briefly argue if Max's strategy is really O(n) or not? If not, what is the time complexity of it?**

No! Max's strategy is O(n^2). The strategy is essentially about sorting a stack using another auxiliary (temporary) stack. It can be described as follows:

```text
1:  Create a temporary stack called tmpStack.
2:  While inputStack is NOT empty:
3:     Pop an element from inputStack and call it temp.
4:     While tempStack is NOT empty and top of tempStack is greater than temp:
5:        Pop from tempStack and push it to the inputStack.
6:     Push temp in tempStack.
7:  The sorted numbers are in tmpStack.
```

When sorting the BookPiles in a BookCart, we must alternate between using one pile as the input stack 
and the other as the temporary stack.

In the worst case where the elements in the input stack are in reverse order, for every iteration 
of the outer loop, all elements currently in temoStack must be pushed back to input stack (inner loop).
The number of times line #5 is executed follows the pattern seen in lecture examples:

$$
1 + 2 + 3 + ... + n = \frac{n(n+1)}{2} \in O(n^2).
$$

**Note to students**: Max's strategy is the solution to the problem of "sort a stack using only 
stack operations and at most one extra stack as auxiliary space". Obviously, we cannot ask this 
question directly since its answer will be a Google search away. The problem, its solution, 
and the **constraints** were conveyed in the context of Max's strategy. Most students managed 
to understand it, some didn't. In particular, some were not able to recognize the constraint of 
"using just one extra stack". Although the point is not explicitly made in the question's description, 
you should have been able to recognize that from the following evidence:

* The fact that the BookCart has exactly two BookPiles (stacks).
* That Max moves books from one pile (stack) to another (back and forth).
* That the examples given only keep "one book" on hold.
    * It is true that in the examples, you didn't need to hold more than one book on 
      hold but if we were to keep more books on hold, that is, to take multiple books off a BookPile, 
      why then move any to the other pile? If we were going to use an additional data structure, 
      we could just remove all the books and put them in an array and sort them. 
      The question hints that we don't want this (it is what Max used to do which was Selection sort).  