package task3;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Represents a BookCart.
 * A book cart contains two "piles" of books.
 */
public class BookCart implements Iterable<Book> {
  protected static final int NUMBER_PILES = 2;
  protected final BookPile[] piles;
  /* Leave the fields as protected; our solution needs them! */

  /**
   * Construct a BookCart.
   */
  public BookCart() {
    piles = new BookPile[NUMBER_PILES];
    piles[0] = new BookPile();
    piles[1] = new BookPile();
  }

  /**
   * Add a book to the top of the pile indicated by its number.
   * @param pile the pile number, as in pile 1 or pile 2.
   *             Pre: pile == 1 || pile == 2
   * @param book a Book to be added to this BookCart.
   *             Pre: book != null
   */
  public void add(int pile, Book book) {
    piles[pile - 1].add(book);
  }

  /**
   * Remove a book from the top of pile indicated by its number.
   * @param pile the pile number, as in pile 1 or pile 2.
   *             Pre: pile == 1 && piles[0].size() > 0 || pile == 2 && piles[1].size() > 0
   * @return a Book removed from this BookCart.
   */
  public Book removeTop(int pile) {
    return piles[pile - 1].removeTop();
  }

  /**
   * Sorts each pile in this BookCart.
   */
  public void sort() {
    // TODO Implement me!
  }

  @Override
  public Iterator<Book> iterator() {
    return new BookCartIterator();
  }

  private class BookCartIterator implements Iterator<Book> {
    private final Iterator<Book> it1 = piles[0].iterator();
    private final Iterator<Book> it2 = piles[1].iterator();
    private Book book1;
    private Book book2;

    @Override
    public boolean hasNext() {
      return it1.hasNext() || it2.hasNext() || book1 != null || book2 != null;
    }

    @Override
    public Book next() {
      updateNextBookFromEachPile();

      Book next;
      if (book1 == null && book2 == null) {
        throw new NoSuchElementException();
      } else if (book1 == null) {
        next = book2;
        book2 = null;
      } else if (book2 == null) {
        next = book1;
        book1 = null;
      } else if (book1.compareTo(book2) <= 0) {
        next = book1;
        book1 = null;
      } else {
        next = book2;
        book2 = null;
      }

      return next;
    }

    private void updateNextBookFromEachPile() {
      if (book1 == null && it1.hasNext()) {
        book1 = it1.next();
      }

      if (book2 == null && it2.hasNext()) {
        book2 = it2.next();
      }
    }
  }
}
