package task3;

/**
 * Represents a BookCart.
 * A book cart contains two "piles" of books.
 */
public class BookCartSln extends BookCart {

  /**
   * Construct a BookCart.
   */
  public BookCartSln() {
    super();
  }

  @Override
  public void sort() {
    sort(piles[0], piles[1]);
    sort(piles[1], piles[0]);
  }

  // Sort the toSort BookPile in O(n) using the spare BookPile as auxiliary space.
  private void sort(BookPile toSort, BookPile spare) {
    if (toSort.size() < 2) {
      return; // no need to sort a pile that has 0 or 1 book
    }

    int moved = 0;
    boolean movedAll = false;
    while (moved > 0 || !movedAll) {
      moved += moveToSpare(toSort, spare, moved);
      if (empty(toSort)) {
        movedAll = true;
      } else {
        Book hold = toSort.removeTop();
        movedAll = empty(toSort);
        moved -= moveToMain(toSort, spare, moved, hold);
        toSort.add(hold);
      }
      moved -= moveToMain(toSort, spare, moved);
    }
  }

  // returns the number of book moved from the "main" to the "spare" BookPile.
  private int moveToSpare(BookPile main, BookPile spare, int moved) {
    if (moved == 0 && !empty(main)) {
      spare.add(main.removeTop()); // move the first book!
      moved++;
    }
    // move books as long as top(main) >= top(spare)
    while (!empty(main) && top(main).compareTo(top(spare)) >= 0) {
      spare.add(main.removeTop());
      moved++;
    }
    return moved;
  }

  // returns the number of book moved from the "spare" to the "main" BookPile.
  private int moveToMain(BookPile main, BookPile spare, int moved, Book hold) {
    int returned = 0;
    // move books as long as spare.top() > hold
    while (moved > 0 && larger(top(spare), hold)) {
      main.add(spare.removeTop());
      moved--;
      returned++;
    }
    return returned;
  }

  // returns the number of books moved from the "spare" to the "main" BookPile.
  private int moveToMain(BookPile main, BookPile spare, int moved) {
    int returned = 0;
    if (empty(main)) {
      main.add(spare.removeTop()); // move back the last book!
      moved--;
      returned++;
    }
    // move books as long as spare.top() <= main.top()
    while (moved > 0 && top(spare).compareTo(top(main)) <= 0) {
      main.add(spare.removeTop());
      moved--;
      returned++;
    }
    return returned;
  }

  // returns true if top > hold or if either are null, false otherwise.
  private boolean larger(Book top, Book hold) {
    return hold == null || top == null || top.compareTo(hold) > 0;
  }

  // make up for BookPile not having top()
  private Book top(BookPile pile) {
    Book book = pile.removeTop();
    pile.add(book);
    return book;
  }

  // make up for BookPile not having empty()
  private boolean empty(BookPile pile) {
    return pile.size() == 0;
  }
}
