package grade;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import task3.Book;
import task3.BookCart;
import task3.BookCartSln;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;

public class BookCartSlnTest {

  protected BookCart cart;

  @BeforeEach
  void setUp() {
    cart = new BookCartSln();
  }

  @Test
  @DisplayName("[1] Sort works when one pile is empty")
  void onePileEmpty() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(5));
    cart.add(1, new Book(3));

    cart.sort();

    int[] sortedCart = {1, 3, 5};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[1] Sort works when both piles are empty")
  void twoPilesEmpty() {
    cart.sort();
    for (Book book : cart) {
      fail();
    }
  }

  @Test
  @DisplayName("[1] Sort works when one pile is sorted already")
  void onePileIsSorted() {
    cart.add(1, new Book(5));
    cart.add(1, new Book(3));
    cart.add(1, new Book(1));
    cart.add(2, new Book(4));
    cart.add(2, new Book(6));
    cart.add(2, new Book(2));
    cart.add(2, new Book(0));

    cart.sort();

    int[] sortedCart = {0, 1, 2, 3, 4, 5, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[1] Sort works when both piles are sorted already")
  void twoPilesAreSorted() {
    cart.add(1, new Book(5));
    cart.add(1, new Book(3));
    cart.add(1, new Book(1));
    cart.add(2, new Book(6));
    cart.add(2, new Book(4));
    cart.add(2, new Book(2));
    cart.add(2, new Book(0));

    // cart.sort();

    int[] sortedCart = {0, 1, 2, 3, 4, 5, 6};
    int index = 0;
    for (Book book : cart) {
      System.out.println(book);
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[1] Sort works when both piles are sorted in reverse order")
  void twoPilesAreReverseSorted() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(3));
    cart.add(1, new Book(5));
    cart.add(2, new Book(0));
    cart.add(2, new Book(2));
    cart.add(2, new Book(4));
    cart.add(2, new Book(6));

    cart.sort();

    int[] sortedCart = {0, 1, 2, 3, 4, 5, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[1] Sort works when one pile is sorted in reverse order")
  void onePileReverseSorted() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(3));
    cart.add(1, new Book(5));
    cart.add(2, new Book(6));
    cart.add(2, new Book(2));
    cart.add(2, new Book(4));
    cart.add(2, new Book(0));

    cart.sort();

    int[] sortedCart = {0, 1, 2, 3, 4, 5, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[2] Sort works when each pile has duplicate values")
  void duplicatesInOnePile() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(3));
    cart.add(1, new Book(3));
    cart.add(2, new Book(6));
    cart.add(2, new Book(4));
    cart.add(2, new Book(4));
    cart.add(2, new Book(0));

    cart.sort();

    int[] sortedCart = {0, 1, 3, 3, 4, 4, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[2] Sort works when the intersection of two piles is non-empty")
  void duplicatesAcrossPiles() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(3));
    cart.add(1, new Book(0));
    cart.add(2, new Book(6));
    cart.add(2, new Book(4));
    cart.add(2, new Book(2));
    cart.add(2, new Book(0));

    cart.sort();

    int[] sortedCart = {0, 0, 1, 2, 3, 4, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

  @Test
  @DisplayName("[1] Sort works for an arbitrary sequence of values in each pile")
  void randomPiles() {
    cart.add(1, new Book(1));
    cart.add(1, new Book(5));
    cart.add(1, new Book(3));
    cart.add(2, new Book(4));
    cart.add(2, new Book(6));
    cart.add(2, new Book(2));
    cart.add(2, new Book(0));

    cart.sort();

    int[] sortedCart = {0, 1, 2, 3, 4, 5, 6};
    int index = 0;
    for (Book book : cart) {
      assertEquals(sortedCart[index++], book.getNumPages());
    }
  }

}