package task2;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public abstract class CumulativeMaxListTest {

  private CumulativeMaxList maxList;

  @BeforeEach
  void setup() {
    maxList = createMaxList();
  }

  protected abstract CumulativeMaxList createMaxList();

  @Test
  @DisplayName("CumulativeMaxList is empty after construction.")
  void emptySequence() {
    assertEquals(0, maxList.length());
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected after adding one element.")
  void sequenceWithOneElement() {
    maxList.insertAt(1, 10);
    assertEquals(1, maxList.length());
    assertEquals(10, maxList.get(1));
    assertEquals(10, maxList.getMaxAt(1));
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected when sequence contains duplicates.")
  void sequenceWithDuplicates() {
    int[] data = {3, 4, 2, 2, 1};
    int[] max = {3, 4, 4, 4, 4};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 1));
      assertEquals(max[i], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected when sequence is one value plastered over.")
  void sequenceSameValue() {
    int[] data = {1, 1, 1, 1, 1};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 1));
      assertEquals(data[i], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected when sequence is sorted in reverse order.")
  void reverseSortedSequence() {
    int[] data = {5, 4, 3, 2, 1};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 1));
      assertEquals(data[0], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected when sequence is sorted.")
  void sortedSequence() {
    int[] data = {1, 2, 3, 4, 5};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 1));
      assertEquals(data[i], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected for a sequence is no particular order.")
  void randomSequence() {
    int[] data = {5, 2, 4, 3, 7, 1};
    int[] max = {5, 5, 5, 5, 7, 7};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 1));
      assertEquals(max[i], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected after inserting at front.")
  void insertAtFront() {
    int[] data = {5, 2, 4, 3, 7, 1};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }

    maxList.insertAt(1, 8);
    assertEquals(8, maxList.get(1));
    assertEquals(8, maxList.getMaxAt(1));

    for (int i = 0; i < data.length; i++) {
      assertEquals(data[i], maxList.get(i + 2));
      assertEquals(8, maxList.getMaxAt(i + 2));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected after inserting in the middle of the sequence.")
  void insertAtMiddle() {
    int[] data = {5, 2, 4, 3, 7, 1};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }
    maxList.insertAt(4, 8);

    int[] max = {5, 5, 5, 8, 8, 8, 8};

    for (int i = 0; i < max.length; i++) {
      assertEquals(max[i], maxList.getMaxAt(i + 1));
    }
  }

  @Test
  @DisplayName("CumulativeMaxList works as expected after inserting at the end.")
  void insertAtEnd() {
    int[] data = {5, 2, 4, 3, 7, 1};

    for (int i = 0; i < data.length; i++) {
      maxList.insertAt(i + 1, data[i]);
    }
    maxList.insertAt(data.length + 1, 8);

    int[] max = {5, 5, 5, 5, 7, 7, 8};

    for (int i = 0; i < max.length; i++) {
      assertEquals(max[i], maxList.getMaxAt(i + 1));
    }
  }
}